<?php

/**
 * @Author: Danoosh Miralayi
 * @Copyright: iPresta.ir
 */
class PackPay extends PaymentPlugin
{
	public $config_fields;
	public $fields_values;
	//public $verify_request;

	public function __construct()
	{
		$this->name = 'packpay';
		$this->version = '1.0';
		$this->author = 'دانوش میرعلایی';
		$this->displayName = 'پکپی';
		$this->description = 'پرداخت از درگاه اینترنتی پکپی';
        $this->ws = 'https://dashboard.packpay.ir/developers/bank/api/v1/purchase';
        $this->ws_verify = 'https://dashboard.packpay.ir/developers/bank/api/v1/purchase/verify';
        $this->ws_type = 'curl';
        $this->redirect_url = 'https://dashboard.packpay.ir/bank/purchase/send?RefId=';
		$this->config_fields = array(
			array(
				'name' => 'client_id',
				'label' => 'client_id',
				'type' => 'text',
			),
			array(
				'name' => 'client_secret',
				'label' => 'client_secret',
				'type' => 'text',
			),
			array(
				'name' => 'refresh_token',
				'label' => 'refresh_token',
				'type' => 'text',
			),
		);
		parent::__construct();
	}
	
	public function setSoapClient($force = false) {
		$this->soapClient =  new stdClass();
	}

	protected function getToken() {
        $data = array (
            'refresh_token'  => $this->fields_values['refresh_token'],
        );
        $data = json_encode($data);
        $header = array(
            'Authorization: Basic '. base64_encode(
                $this->fields_values['client_id'].':'.$this->fields_values['client_secret']
            ),
        );

        $ws_token = 'https://dashboard.packpay.ir/oauth/token?grant_type=refresh_token&refresh_token='
            .$this->fields_values['refresh_token'];

        $curl_res = $this->curl_webservice($ws_token,'POST', $data, $header, array(), true);
        $result 	= json_decode($curl_res['data']);
        if ($curl_res['status'] != 200) {
            $this->errors[] = 'خطا در برقرای ارتباط و دریافت توکن. تنظیمات پکپی را کنترل نمایید.';
            $this->errors[] = $curl_res['status'].': '.$result->error;
            return false;
        }
        return $result->access_token;
	}


	public function payRequest($params)
	{
		$token = $this->getToken();
		if(!$token) {
		    return false;
        }

	    $data = array ();
		$this->ws .= '?amount='.$params['amount'];
		$this->ws .= '&callback_url='.urlencode($params['back_url']);
        $header = array(
				'Accept: application/json',
				'Content-Type: application/json',
				'Authorization: bearer '.$token,
		);

		$curl_res = $this->curl_webservice($this->ws,'POST', $data, $header, array());
        $result 	= json_decode($curl_res);

		if(!is_object($result)) {
			$this->errors[] = empty($curl_res) ? 'Connection Error' : $curl_res;
			return false;
		}
		if (isset($result->status) && $result->status != "0") {
			$this->errors[] = 'خطا در دریافت شناسه پرداخت. دوباره تلاش کنید.';
			$this->errors[] = $result->status. ': '.$result->message;
			return false;
		}
		
		if (isset($result->status) && $result->status == "0")
		{
			$this->setCookie("reference_code", $result->reference_code);
            $this->reference_number = $result->reference_code;

			$this->redirect_url .= $result->reference_code;
			return true;
		}
		else {
			$this->errors[] = 'خطای نامشخص';
			return false;
		}
	}

	public function verifyRequest($params)
	{

        $token = $this->getToken();
        if(!$token) {
            return false;
        }

        $reference_code = Tools::getValue('reference_code');
		$this->reference_number = $reference_code;
		$data = array ();
        $this->ws_verify .= '?reference_code='. $reference_code;
		$header = array(
				'Accept: application/json',
				'Content-Type: application/json',
				'Authorization: bearer '.$token,
		);

        $curl_res = $this->curl_webservice($this->ws_verify,'POST', $data, $header, array());
        $result 	= json_decode($curl_res);

        if(!is_object($result)) {
            $this->errors[] = empty($curl_res) ? 'Connection Error' : $curl_res;
            return false;
        }
        if (isset($result->status) && $result->status != "0") {
            $this->errors[] = 'خطا در تایید تراکنش';
            $this->errors[] = $result->status. ': '.$result->message;
            return false;
        }

        if (isset($result->status) && $result->status == "0") {
            return true;
        }
        else {
            $this->errors[] = 'خطای نامشخص';
            return false;
        }
	}
}